"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = require("chalk");
const child_process_1 = require("child_process");
const fs_extra_1 = require("fs-extra");
const jovo_cli_core_1 = require("jovo-cli-core");
const Listr = require("listr");
const path_1 = require("path");
const rimraf = require("rimraf");
const util_1 = require("util");
const utils_1 = require("../utils");
const Prompts_1 = require("../utils/Prompts");
const execAsync = util_1.promisify(child_process_1.exec);
const rimrafAsync = util_1.promisify(rimraf);
class Update extends command_1.Command {
    async run() {
        try {
            this.parse(Update);
            const tasks = new Listr([], {
                renderer: new utils_1.JovoCliRenderer(),
                collapse: false,
            });
            const project = jovo_cli_core_1.getProject();
            await project.init();
            const packageVersions = await utils_1.getPackageVersionsNpm(/^jovo\-/);
            const outOfDatePackages = [];
            if (Object.keys(packageVersions).length > 0) {
                this.log('Jovo packages of current project:');
                for (const [name, pkg] of Object.entries(packageVersions)) {
                    let text = `  ${name}: ${pkg.local}`;
                    if (pkg.local !== pkg.npm) {
                        outOfDatePackages.push(name);
                        text += chalk_1.default.grey(`  -> ${pkg.npm}`);
                    }
                    this.log(text);
                }
            }
            if (outOfDatePackages.length === 0) {
                this.log('\n\nAll packages are already up to date!\n');
                return;
            }
            const { update } = await Prompts_1.promptUpdateVersions(outOfDatePackages.length);
            if (update !== Prompts_1.ANSWER_UPDATE) {
                this.exit();
            }
            let npmUpdateOutput = '';
            for (let i = 0; i < outOfDatePackages.length; i++) {
                outOfDatePackages[i] = outOfDatePackages[i] + '@latest';
            }
            tasks.add({
                title: 'Updating Jovo packages...',
                task: async () => {
                    const updateCommand = `npm install ${outOfDatePackages.join(' ')} --loglevel=error`;
                    try {
                        const { stdout, stderr } = await execAsync(updateCommand, {
                            cwd: project.getProjectPath(),
                        });
                        npmUpdateOutput = stdout;
                        if (stderr) {
                            throw new Error(stderr);
                        }
                    }
                    catch (err) {
                        this.error(err);
                    }
                },
            });
            const bundleDirectoryPath = project.getZipBundleDirectoryPath();
            try {
                fs_extra_1.statSync(bundleDirectoryPath);
                const bundleNodeDirectoryPath = path_1.join(bundleDirectoryPath, 'node_modules');
                fs_extra_1.statSync(bundleNodeDirectoryPath);
                tasks.add({
                    title: 'Deleting "node_modules" in bundle directory...',
                    async task() {
                        await rimrafAsync(bundleNodeDirectoryPath);
                    },
                });
            }
            catch (err) { }
            await tasks.run();
            this.log();
            this.log('Update completed.');
            this.log();
            this.log('Update output: ');
            this.log('-------------------');
            npmUpdateOutput ? this.log(npmUpdateOutput) : this.log('Everything is up to date!');
            this.log('Changelog: https://raw.githubusercontent.com/jovotech/jovo-framework/master/CHANGELOG.md');
        }
        catch (err) {
            this.error(`There was a problem:\n${err}`);
        }
    }
}
exports.Update = Update;
Update.description = 'Updates the Jovo modules in project to latest versions.';
Update.examples = ['jovo update'];
//# sourceMappingURL=update.js.map