"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jovo_cli_core_1 = require("jovo-cli-core");
const jovo_cli_platform_alexa_1 = require("jovo-cli-platform-alexa");
const jovo_cli_platform_google_1 = require("jovo-cli-platform-google");
const AVAILABLE_PLATFORMS = [jovo_cli_platform_alexa_1.JovoCliPlatformAlexa, jovo_cli_platform_google_1.JovoCliPlatformGoogle];
const instances = {};
const project = jovo_cli_core_1.getProject();
function getAllAvailable() {
    return AVAILABLE_PLATFORMS.map((platform) => platform.PLATFORM_KEY);
}
exports.getAllAvailable = getAllAvailable;
function get(platform) {
    if (!instances.hasOwnProperty(platform)) {
        instances[platform] = createPlatformInstance(platform);
    }
    return instances[platform];
}
exports.get = get;
function getAll(platform, stage) {
    if (platform) {
        return [platform];
    }
    const projectPlatforms = [];
    const config = project.getConfig(stage);
    for (const platformName of getAllAvailable()) {
        const platformInstance = get(platformName);
        try {
            if (config.hasOwnProperty(platformName)) {
                projectPlatforms.push(platformName);
            }
        }
        catch (err) {
            if (platformInstance.hasPlatform()) {
                projectPlatforms.push(platformName);
            }
        }
    }
    return projectPlatforms;
}
exports.getAll = getAll;
function createPlatformInstance(name) {
    for (let i = 0; i < AVAILABLE_PLATFORMS.length; i++) {
        if (AVAILABLE_PLATFORMS[i].PLATFORM_KEY === name) {
            return new AVAILABLE_PLATFORMS[i]();
        }
    }
    throw new Error(`The platform "${name}" is not supported!`);
}
function addCliOptions(command, options) {
    for (const platform of getAllAvailable()) {
        const instance = get(platform);
        instance.getAdditionalCliOptions(command, options);
    }
}
exports.addCliOptions = addCliOptions;
function validateCliOptions(command, options) {
    for (const platform of getAllAvailable()) {
        const instance = get(platform);
        if (!instance.validateAdditionalCliOptions(command, options)) {
            return false;
        }
    }
    return true;
}
exports.validateCliOptions = validateCliOptions;
//# sourceMappingURL=Platforms.js.map